@@ -6,12 +6,14 @@ use Config;
 
 my $Perl = $^X;
 
-BEGIN { plan tests => 18 }
+BEGIN { plan tests => 32 }
 
 use Expect;
 #$Expect::Exp_Internal = 1;
 #$Expect::Debug = 1;
 
+print "\nBasic tests...\n\n";
+
 {
   my $exp = Expect->spawn("$Perl -v");
   ok(defined $exp);
@@ -21,6 +23,8 @@ use Expect;
   ok(not $exp->expect(3, "Copyright"));
 }
 
+print "\nTesting exec failure...\n\n";
+
 {
   my $exp = new Expect;
   ok(defined $exp);
@@ -36,6 +40,79 @@ use Expect;
 #  ok(defined $res and $res == 1);
 }
 
+print "\nTesting exp_continue...\n\n";
+
+{
+  my $exp = new Expect($Perl . q{ -e 'foreach (qw(A B C D End)) { print "$_\n"; }' });
+  my $state = "A";
+  $exp->expect(2,
+	       [ "[ABCD]" => sub { my $self = shift;
+				   ok($self->match eq $state);
+				   $state++;
+				   exp_continue;
+				 } ],
+	       [ "End" => sub { ok($state eq "E"); } ],
+               [ eof => sub { print "EOF\n"; ok(0); } ],
+               [ timeout => sub { print "TIMEOUT\n"; ok(0);} ],
+              );
+  $exp->hard_close();
+}
+
+{
+  my $exp = new Expect($Perl . q{ -e 'print "Begin\n"; sleep (5); print "End\n";' });
+  my $cnt = 0;
+  $exp->expect(1,
+	       [ "Begin" => sub { ok(1); exp_continue; } ],
+	       [ "End" => sub { ok(1); } ],
+               [ eof => sub { print "EOF\n"; ok(0); } ],
+               [ timeout => sub { $cnt++; ($cnt < 7)? exp_continue : 0;} ],
+              );
+  ok($cnt > 2 and $cnt < 7);
+  $exp->hard_close();
+}
+
+print "\nTesting -notransfer...\n\n";
+
+{
+  my $exp = new Expect($Perl . q{ -e 'print "X some other\n"; sleep 5;'});
+  $exp->notransfer(1);
+  $exp->expect(3,
+	       [ "some" => sub { ok(1); } ],
+               [ eof => sub { print "EOF\n"; ok(0); } ],
+               [ timeout => sub { print "TIMEOUT\n"; ok(0);} ],
+              );
+  $exp->expect(3,
+	       [ "some" => sub { ok(1); } ],
+               [ eof => sub { print "EOF\n"; ok(0); } ],
+               [ timeout => sub { print "TIMEOUT\n"; ok(0);} ],
+              );
+  $exp->expect(3,
+	       [ "other" => sub { ok(1); } ],
+               [ eof => sub { print "EOF\n"; ok(0); } ],
+               [ timeout => sub { print "TIMEOUT\n"; ok(0);} ],
+              );
+  sleep(6);
+  $exp->expect(3,
+	       [ "some" => sub { my $self = shift; ok(1); $self->set_accum($self->after()); } ],
+               [ eof => sub { print "EOF\n"; ok(0); } ],
+               [ timeout => sub { print "TIMEOUT\n"; ok(0);} ],
+              );
+  $exp->expect(3,
+	       [ "some" => sub { ok(0); } ],
+	       [ "other" => sub { my $self = shift; ok(1); $self->set_accum($self->after()); } ],
+               [ eof => sub { print "EOF\n"; ok(0); } ],
+               [ timeout => sub { print "TIMEOUT\n"; ok(0);} ],
+              );
+  $exp->expect(3,
+	       [ "some" => sub { ok(0); } ],
+	       [ "other" => sub { ok(0); } ],
+               [ eof => sub { print "EOF\n"; ok(1); } ],
+               [ timeout => sub { print "TIMEOUT\n"; ok(0);} ],
+              );
+}
+
+print "\nTesting raw reversing...\n\n";
+
 {
   my @Strings =
     (
@@ -80,6 +157,8 @@ use Expect;
 ------------------------------------------------------------------------------
 _EOT_
 
+# we check if the raw pty can handle large chunks of text at once
+
   my $randstring = 'fakjdf ijj845jtirg8e 4jy8 gfuoyhjgt8h gues9845th guoaeh gt98hae 45t8u ha8rhg ue4ht 8eh tgo8he4 t8 gfj aoingf9a8hgf uain dgkjadshftuehgfusand987vgh afugh 8h 98H 978H 7HG zG 86G (&g (O/g &(GF(/EG F78G F87SG F(/G F(/a sldjkf hajksdhf jkahsd fjkh asdHJKGDSGFKLZSTRJKSGOSJDFKGHSHGDFJGDSFJKHGSDFHJGSDKFJGSDGFSHJDGFljkhf lakjsdh fkjahs djfk hasjkdh fjklahs dfkjhasdjkf hajksdh fkjah sdjfk hasjkdh fkjashd fjkha sdjkfhehurthuerhtuwe htui eruth ZI AHD BIZA Di7GH )/g98 9 97 86tr(& TA&(t 6t &T 75r 5$R%/4r76 5&/% R79 5 )/&';
   my $maxlen;
   $exp->log_stdout(0);
@@ -114,12 +193,17 @@ _EOT_
   ok($maxlen > 80);
 }
 
+# Now test for the max. line length. Some systems are limited to ~255
+# chars per line, after which they start loosing characters.  As Cygwin 
+# then hangs and cannot be freed via alarm, we only test up to 250 characters
+# to avoid that.
+
 {
   my $exp = new Expect ("$Perl -ne 'chomp; sleep 0; print scalar reverse, \"\\n\"'")
     or die "Cannot spawn $Perl: $!\n";
 
   $exp->log_stdout(0);
-  my $randstring = 'Fakjdf ijj845jtirg8e 4jy8 gfuoyhjgt8h gues9845th guoaeh gt98hae 45t8u ha8rhg ue4ht 8eh tgo8he4 t8 gfj aoingf9a8hgf uain dgkjadshftuehgfusand987vgh afugh 8h 98H 97BH 7HG zG 86G (&g (O/g &(GF(/EG F78G F87SG F(/G F(/a sldjkf hajksdhf jkahsd fjkh asdljkhASDJHARZF2345fegzhuLASDLKASHDJAHADjkahsdaSAHKHASKDHAKSHDAJKSHDf lakjsdh fkjahs djfk hasjkdh fjklahs dfkjhasdjkf hajksdh fkjah sdjfk hasjkdh fkjashd fjkha sdjkfhehurthuerhtuwe htui eruth ZI AHD BIZA Di7GH )/g98 9 97 86tr(& TA&(t 6t &T 75r 5$R%/4r76 5&/% R79 5 )/8';
+  my $randstring = 'Fakjdf ijj845jtirg8e 4jy8 gfuoyhjgt8h gues9845th guoaeh gt98hae 45t8u ha8rhg ue4ht 8eh tgo8he4 t8 gfj aoingf9a8hgf uain dgkjadshftuehgfusand987vgh afugh 8h 98H 97BH 7HG zG 86G (&g (O/g &(GF(/EG F78G F87SG F(/G F(/a slkf ksdheq@f jkahsd fjkh%&/"#w';
   my $maxlen;
   my $exitloop;
   foreach my $len (1 .. length($randstring)) {
@@ -132,7 +216,7 @@ _EOT_
     };
     if ($@) {
       ok($maxlen > 80);
-      print "Warning: your default pty blocks when sending more than $maxlen bytes!\n";
+      print "Warning: your default pty blocks when sending more than $maxlen bytes per line!\n";
       $exitloop = 1;
       last;
     }
@@ -143,12 +227,12 @@ _EOT_
 		 [ eof => sub { ok(0); die "EOF"; } ],
 		);
   }
-  print "Good, your default pty can handle at least ".length($randstring)." bytes at a time.\n" if not $exitloop;
-  ok($maxlen > 80);
+  print "Good, your default pty can handle lines of at least ".length($randstring)." bytes at a time.\n" if not $exitloop;
+  ok($maxlen > 100);
 }
 
 {
-  print "Testing controlling terminal...\n";
+  print "\nTesting controlling terminal...\n\n";
   my $exp = new Expect($Perl . q{ -MIO::Handle -e 'open(TTY, "+>/dev/tty") or die "no controlling terminal"; autoflush TTY 1; print TTY "prompt: "; $s = <TTY>; chomp $s; print "uc: \U$s\n"; close TTY; exit 0;'});
 
   my $pwd = "pAsswOrd";
@@ -174,7 +258,7 @@ _EOT_
 	      );
 }
 
-print "Checking if exit status is returned correctly...\n";
+print "\nChecking if exit status is returned correctly...\n\n";
 
 {
   my $exp = new Expect($Perl . q{ -e 'print "pid: $$\n"; sleep 2; kill 3, $$;'});
@@ -189,7 +273,7 @@ print "Checking if exit status is returned correctly...\n";
   ok(($status & 0x7F) == 3);
 }
 
-print "Checking if EOF on pty slave is correctly reported to master...\n";
+print "\nChecking if EOF on pty slave is correctly reported to master...\n\n";
 
 {
   my $exp = new Expect($Perl . q{ -e 'close STDIN; close STDOUT; close STDERR; sleep 4;'});
@@ -201,31 +285,3 @@ print "Checking if EOF on pty slave is correctly reported to master...\n";
 }
 
 exit(0);
-
-{
-  my $exp = new Expect($Perl . q{ -e 'print "some string\n"; sleep 5;'});
-  $exp->notransfer(1);
-  $exp->expect(3,
-	       [ qr/some string/ => sub { ok(1); } ],
-               [ eof => sub { print "EOF\n"; ok(0); } ],
-               [ timeout => sub { print "TIMEOUT\n"; ok(0);} ],
-              );
-  $exp->expect(3,
-	       [ qr/some string/ => sub { ok(1); } ],
-               [ eof => sub { print "EOF\n"; ok(0); } ],
-               [ timeout => sub { print "TIMEOUT\n"; ok(0);} ],
-              );
-  sleep(6);
-  $exp->expect(3,
-	       [ qr/some string/ => sub { ok(1); } ],
-               [ eof => sub { print "EOF\n"; ok(1); } ],
-               [ timeout => sub { print "TIMEOUT\n"; ok(0);} ],
-              );
-  $exp->expect(3,
-	       [ qr/some string/ => sub { ok(0); } ],
-               [ eof => sub { print "EOF\n"; ok(1); } ],
-               [ timeout => sub { print "TIMEOUT\n"; ok(0);} ],
-              );
-}
-
-exit(0);
